/*******************************************************************************
     CHAMA GAROM          
     (Verso em C - Use compilador free 'WinAvr - verso 2005')
     
     Painel Eletrnico de Chamada de Garom
     
     Microcontrolador: ATMega8
	 
     Autor: Cludio Lrios

     Data: 24/07/2024 (esta verso em C)
     (obs. primeira verso feita em asm em  19/11/2012)
 
     Objetivo:  Painel com 6 dgitos para indicar as chamadas de garons feitas
    atravs de controle remoto situado na mesa, com 2 interruptores: 1 'chama'
     e o outro 'cancela' a chamada. Os 2 digitos da esquerda mostrar o cliente
     de maior prioridade, seguido pelos prximos 4 dgitos. Os 2 dgitos mais a
     direita, alm de mostrar a sequncia de prioridade, mostrar novos chamados
     de garom de forma momentana (piscando e gerando um som). Em seguida,
     retorna mostrando a sequncia de atendimento. Cada vez  que for atendido um
     dos 3 prioritrios, a fila se desloca para a esquerda, apresentando o novo
     nmero da mesa na fila de espera (se houver, seno apresentar '---' nos
     displays).
     Capacidade para memorizar at 200 controles.
     Fila sequencial na memria para 100 mesas.
     Display multiplexado.
     Protocolo usado nos transmissores das mesas so da tecnologia HT6P20B.
     O painel de display pode ser confeccionado com leds individuais ou adquirir
     display 7 seguimentos gigantes.
     Obs.Fila fica alojada na memria Ram. Dever ser provido uma fonte de 
	 energia para o microcontrolador independente da rede, para no perder
	 dados em caso de falta de energia na rede eltrica pblica.
;
    up/del=modo programao
	up/del/del_all =deletar eeprom inteira (conforme jumper: colocado = interna; retirado = externa 24c08)
    del=deleta controle salvo de forma errada
     Blog PicSource
     
     Uso didtico

*******************************************************************************/
 //includes
 #include <inttypes.h>
 #include <avr/io.h>
 #include <compat/twi.h>

 //DEFINIES
 #define QBIT 28 
 #define F_CPU 8000000UL
 #define tmax 80
 #define  rfin     0 //PINB0 pino 14
 #define  prog_res 6 //PINB6 pino 9
 #define  up       4 //PINB4 pino 18
 #define  del      3 //PINB3 pino 17
 #define  del_all 1 //PINB1 pino 15
 
 //#define catodo_comum 1 //descomente para nodo comum
 
//pinos

/* pinos usados no seguimentos dos displays
#define sega PORTD,0;//a
#define segb PORTD,1;//b
#define segc PORTD,2;//c
#define segd PORTD,3;//d
#define sege PORTD,4;//e
#define segf PORTD,5;//f
#define segg PORTD,6;//g
*/

#define d7disp5  _BV(PD7);
#define b2disp4  _BV(PB2);
#define c3disp3  _BV(PC3);
#define c2disp2  _BV(PC2);
#define c1disp1  _BV(PC1);
#define c0disp0  _BV(PC0);

//constantes para converso 7 seguimentos

//                         PGFEDCBA   SEGUIMENTOS
//                   PORTD 76543210
 const uint8_t num[] = {0B00111111,//0
                         0B00000110,//1
                         0B01011011,//2
                         0B01001111,//3
                         0B01100110,//4
                         0B01101101,//5
                         0B01111101,//6
                         0B00000111,//7
                         0B01111111,//8
                         0B01101111,//9
                         0B01110111,//A
                         0B01111100,//B
                         0B00111001,//C
                         0B01011110,//D
                         0B01111001,//E
                         0B01110001,//F
                         0B00111101,//G  (16)
                         0B01110110,//H  (17)
                         0B00000110,//i  (18)
                         0B00111000,//L  (19)
                         0B01010100,//n  (20)
                         0B00111111,//O  (21)
                         0B01110011,//P  (22)
                         0B00110001,//R  (23)
                         0B01101101,//S  (24)
                         0B00111110,//U  (25)
                         0B01000000,//'-' (26)
                         0B00000000,//' ' (27)
						 0B00110111,//'M' (28)
                                        };
                                        
 
 uint8_t  chamados[100];//armazena os nmeros de mesas que pediram garom
 
 
 uint8_t  dig0,dig1,dig2,dig3,dig4,dig5;
 uint8_t  lc,hc,qb,vez;
 uint8_t  address,address1;
 uint32_t  buffer,buffer1,cnt_trava_mc;
 uint16_t  address_mem;//endereo de salvamento para EEprons
 uint16_t  pont_fila;
 uint8_t   num_mesa;
 uint8_t   pos_fila;
 //Estrutura para bits usados nos flags
struct{
    unsigned flag_rok:1;//(LSB)1=recebeu uma transmisso Ht6p20b
	unsigned flag_h:1; //1=recebeu bit inicial do protocolo Ht6p20b aps intervalo
	unsigned flag_nl:1;//1= recebeu a segunda transmisso para comparao 
	unsigned flag_mp:1;//1= modo de programao dos tx das mesas 
	unsigned flag_in:1;//1= sinaliza que boto de chamada de garom foi apertado 
	unsigned flag_out:1;//1= sinaliza que boto de atendimento concluido foi apertado  
    unsigned flag_nm:1; 
	unsigned flag_x4:1;//(MSB) 
 }flags;

 
/*******************************************************************************
    PROTTIPOS
*******************************************************************************/
int main(void);
void programar_mem_int(void);
void programar_mem_ext(void);
void entrar_fila(void);
void mostrar_fila(void);
uint8_t procura_se_ja_na_fila(uint8_t q_mesa);
void recebe(void);
void reset_rx (void);
void salvar_tx_em_EEprom(void);
void testa_bt(void);
void EEPROM_Write(uint16_t address,uint8_t edado);
uint8_t EEPROM_Read(uint16_t address);
void EEPROM_24C08_Init(void);
void EEPROM_24C08_WrSingle(uint16_t wAddr, uint8_t wData);
uint8_t EEPROM_24C08_RdSingle(uint16_t rAddr);
void TWI_Start(void);
void TWI_Write(uint8_t dataddress);
uint8_t TWI_Busy(void);
uint8_t TWI_Read(uint8_t b);
void TWI_Stop(void);
void multiplexa(void);
void dly_us(uint8_t utime);
void dly_ms(uint8_t mtime);
void dly_msg(void);
void dly_pisca_disp(void);
void dly_tom1(void);
void dly_tom2(void);
void dly_tom3(void);
void dly_write(void);
uint8_t controle_foi_cadastrado(void);

/*******************************************************************************
    Rotina Principal
*******************************************************************************/
int main() {
  char aux,c;
  
   DDRB = 0B00100100;                       // Set signal port as output/inputs
   DDRC = 0B111111;                        // Set signal port as output
   DDRD = 0xFF;                            // Set signal port as output

   PORTB = 0b01011000;                           // Clear signal ports
   PORTC = 0x00;
   PORTD = 0x00;
   EEPROM_24C08_Init();    //inicializa comunicao com EEprom Externa 24C08
  // TCCR1B = 1;    //divide 1  prescaller
  //   TOIE1_bit=1; //liga interrupo do timer 1 over flow
  // SREG_I_bit = 1;  // Liga geral das interrupes
   
   dig0=26;    //mostra traos no displays
   dig1=26;
   dig2=26;
   dig3=26;
   dig4=26;
   dig5=26;
   //recepo
   flags.flag_in=0;
   flags.flag_out=0;
   flags.flag_rok=0;
   flags.flag_h=0;
   flags.flag_nl=0;
   flags.flag_mp=0;	
   flags.flag_nm=0;	
   qb=QBIT;
   lc=0;
   hc=0;
   num_mesa=0xff;
   address_mem=0;
   pont_fila=0;//ponteiro da fila inicializado
   pos_fila=0xff;//posio de fila inicializada
   cnt_trava_mc=0;
   for(c=0;c<100;c++) chamados[c]=0;//pem traos '0'na fila de espera   
   aux=0;//-teste
/*******************************************************************************
    Lao principal
*******************************************************************************/
 for(;;){
 
     if(cnt_trava_mc)cnt_trava_mc--;  // decrementa tempo para trava prog mc
     testa_bt();
     recebe();
     multiplexa();
	 
	 
     if(flags.flag_rok){
       if(!flags.flag_nl){ buffer1=buffer; flags.flag_nl=1; }  //foi a primeira recepo
       else{  //foi a segunda recepo

            flags.flag_nl=0;
            if(buffer==buffer1){ //compara as recepes se iguais		      
		 
               if(flags.flag_mp){  //se flag de modo de programao setado
                salvar_tx_em_EEprom();
               }
               else{			  
                  entrar_fila();//entra na fila de atendimento dos garons				 
               }               
		  	
        } //if(!flags.flag_nl)         
		    
        multiplexa();           
        
      }
     }//if(flags.flag_rok){	 
	 
     flags.flag_rok=0;
     mostrar_fila();//mostra nos displais as mesas em atendimento         
       
 }//for(;;)

}//void main()


/*******************************************************************************
    Sub-Rotinas Gerais
*******************************************************************************/
//==============================================================================
//          Rotina para Teste de Botes
//==============================================================================

void testa_bt(void){
   #define TEMPO_INCREMENTO_RAPIDO 3000
   #define TEMPO_INCREMENTO 4000
   #define TEMPO_MODO_PROG  65535 // aprox. 5 segundos
   #define QUANT_RAPIDO 5
   #define TEMPO_P_APAGAR 65000

  static uint8_t vel;
  static uint16_t ct0;
  static uint16_t ct1;
  static uint16_t ct2=0;
  uint16_t end;
  uint8_t dig1a,dig0a,j;
  
  if( flags.flag_mp && (!(PINB & (1 <<del_all)))){//se boto 'delete tudo das memrias' foi apertado
    ct2++;
	if(ct2 > TEMPO_P_APAGAR){
	  ct2=0;
	  pont_fila=0;//zera ponteiro da fila
	  for(end=0;end<99;end++){//apagar fila de chamadas
	    chamados[end]=0;
	  }  
	  
	  
	  if(!(PINB & (1 << prog_res))){//apaga EEprom interna (sem jumper 'prog_res')
	    dig5=10; dig4=22;dig3=16;dig2=18;dig1=27; dig0=27;
		dly_msg();//delay para apresentar a mensagem APGI
	    for(end=0;end<300;end++){		 
		   if(EEPROM_Read(end) != 0xff){
		     multiplexa();
		     EEPROM_Write(end,0xff);
		   }
		}
	  
	  }
	  else{//apaga EEprom externa (com jumper 'prog_res')
	     dig5=10; dig4=22;dig3=16;dig2=14;dig1=27; dig0=27;
		dly_msg();//delay para apresentar a mensagem APGE
	     for(end=0;end<300;end++){
		   if(EEPROM_24C08_RdSingle(end) != 0xff){
		     multiplexa();
		     EEPROM_24C08_WrSingle(end,0xff); //escreve na EEprom Externa 24C08
		   }
		}
	  
	  }
	  dig0=25;//'U'    
      dig1=0; //'0'
      dig2=16;//'G'
      dig3=10;//'A'
      dig4=22;//'P'
      dig5=10;//'A'
	  dly_msg();//delay para mostrar msg 'APAGOU '
	  flags.flag_mp=0;//sa do modo de programao	
    }
  }
  
//-------------------------------------
//Se flags.mp for igual a '1' 
 if(flags.flag_mp){//=1
      if((!(PINB & 1 << up)) && (!(PINB & 1<< del)))return;//espera soltar botes 'del e 'up'    
      ct0=0;	 
	  ct1++;
	  
	  if(ct1>=TEMPO_INCREMENTO){	 
        ct1=0;
	    if(!(PINB & 1 << up)) {//se 'up' =0	  
	      vel++;
		  if(vel>QUANT_RAPIDO) {vel--; ct1=TEMPO_INCREMENTO_RAPIDO;}		
          dig0++;
          if(dig0>9){
            dig0=0;
            dig1++;
            if(dig1>9)dig1=0;
			
           }
        }
        else{vel=0;}	
        
        if((!(PINB & 1<< del)) && (PINB & 1 << up)){
		   dly_msg();//tempo mnimo que boto tem que ficar apertado
		  if(!(PINB & 1<< del)){
		    dig2=13;//'D'
		    dly_msg();//delay para mostrar msg 'PROGD'
		    while(!(PINB & 1<< del)){//aguarda solta boto 'del'
		      multiplexa();
			  dly_ms(4);
		    }
		    end=0;
		    dig1a=dig1;
		    dig0a=dig0;
		    if(dig0a)dig0a--;//ajusta 1 unidade a menos (mesa 1 posio 0)
		    while(dig1a > 0){end = end + 10; dig1a--;}
		    end = end + dig0a;
		    end = end * 3;
		    if(!(PINB & (1 << prog_res))){//apaga EEprom interna (com jumper 'prog_res')
		      for(j=0;j<3;j++){
		        if(EEPROM_Read(end+j) != 0xff){
		          multiplexa();
		          EEPROM_Write((end+j),0xff);
		        }
			 }
            }
		    else{
		     for(j=0;j<3;j++){
		       if(EEPROM_24C08_RdSingle(end+j) != 0xff){
		         multiplexa();
		         EEPROM_24C08_WrSingle((end+j),0xff); //escreve na EEprom Externa 24C08
		       }
		     }
		    }		  
		 
		    dig0=25;//'U'    
            dig1=0; //'0'
            dig2=16;//'G'
            dig3=10;//'A'
            dig4=22;//'P'
            dig5=10;//'A'
		    dly_msg();//delay para mostrar msg 'APAGOU '
		    pont_fila=0;//zera ponteiro da fila
	        for(end=0;end<99;end++){//apagar fila de chamadas
	          chamados[end]=0;
	        } 
		    flags.flag_mp=0;//sa do modo de programao	
		  }
        }		
		
      }//if(ct1>=TEMPO_INCREMENTO){
     }//if(flags.flag_mp){
	 //----------------------------------------------
	 //Se flags.flag_mp for igual a '0'
     else{//==0
      if(!(PINB & 1 << up) && !(PINB & 1 << del)){
          if(++ct0==TEMPO_MODO_PROG){
             flags.flag_mp=1;  //entra em modo de programao
             dig0=1;    
             dig1=0;
             dig2=26;//'-'
             dig3=28;//'M'
             dig4=16;//'G'
             dig5=22;//'P'
             ct0=0;
             address_mem=0;

           }//if(ct==65535){
        } //if((!(PINB & 1 << up)) && (!(PINB & 1<< del))){
        else {ct0=0; ct1=0;}//else->if(!(PINB & 1 << up) && !(PINB & 1 << del)){
     } // else->if(flags.flag_mp){
}//testa_bt(void){

//==============================================================================
//    Rotina para Programar TX da mesa na EEprom
//    (Conforme posio do Jumper ser na EEprom interna ou externa)
//==============================================================================

void salvar_tx_em_EEprom(void){
    uint8_t a;
    a=dig1;//obtem a dezena	
    if(cnt_trava_mc)return;  //trava por tempo para no gravar mesmo controle
    address_mem = 0;
    while(a>0){a--; address_mem = address_mem+10;} //obtem endereo 
    address_mem = address_mem + dig0;   //a ser gravado
	address_mem--;//------------------------------------------------------------------xxxxx
    address_mem = address_mem * 3; //multiplica por 3 (3 posies por controle)
    if(!(PINB & (1<<prog_res))){ //EEprom interna para controles normais (com jumper)
	
      EEPROM_Write(address_mem,buffer>>16);
      EEPROM_Write(address_mem+1,buffer>>8);
      EEPROM_Write(address_mem+2,buffer);
    }
    else{  //EEprom enterna para controles reservas (sem jumper)
	
	  EEPROM_24C08_WrSingle(address_mem,buffer>>16);
      EEPROM_24C08_WrSingle(address_mem+1,buffer>>8);
      EEPROM_24C08_WrSingle(address_mem+2,buffer);
    }
    dig0++;
    if(dig0>9){dig0=0;dig1++;}
    if(dig1>9)dig1=0;  //limita a 99 controles como mximo
    cnt_trava_mc=0x3fff;  //aprox. 1 segundo entre controle a ser gravado
}


void EEPROM_Write(uint16_t address,uint8_t edado){
  while(EECR & (1 << EEWE));//aguarda completar previa escrita
  EEAR = address;//coloca o endereo a ser gravado
  EEDR = edado;//coloca o dado a ser gravado
  EECR |= (1 << EEMWE);//seta EEMWE
  EECR |= (1 << EEWE); //seta EEWE
}


uint8_t EEPROM_Read(uint16_t address){
  while(EECR & (1 << EEWE));//aguarda completar previa escrita
  EEAR = address;//coloca o endereo a ser lido
  EECR |= (1 << EERE); //seta EERE
  return EEDR;
}

//==============================================================================
//          Rotina para Colocar o Nmero da Mesa na Fila de Espera
//==============================================================================

void entrar_fila(void){  
  
  num_mesa= controle_foi_cadastrado();//retorna '0xff' para no e nmero 1 a 99 para cadastrados
  if(num_mesa==0xff){
    flags.flag_in=0;
    flags.flag_out=0;    
    return;//no tem este controle cadastrado nas eeprons
   }
   
  pos_fila=(procura_se_ja_na_fila(num_mesa));//pos_fila=0xff no encontrado na fila ; pos_fila=xx est nesta posio da fila
  flags.flag_nm=0;
  
  if(flags.flag_out && pos_fila==0xff){//boto 'atendido' e no cadastrado sai da rotina
     flags.flag_in=0;
	 flags.flag_out=0;
	 flags.flag_nm=0;// no ir mostrar e piscar na tela
	 num_mesa=0xff;
	 return;
  }
   if(flags.flag_in && pos_fila==0xff){//coloca mesa na fila pois falta atender
     chamados[pont_fila]=num_mesa;
	 pont_fila++;
	 if(pont_fila>99)pont_fila--;//limita, mas com erro de sobreescrita do cliente 99
      flags.flag_in=0;
	  flags.flag_out=0;
	  flags.flag_nm=1;// ir mostrar e piscar na tela
	  return;
   } 
}

//==============================================================================
//   ROTINA PARA PROCURAR SE MESA J ESTA NA FILA DE ESPERA
//==============================================================================

uint8_t procura_se_ja_na_fila(uint8_t q_mesa){
 uint8_t d;
  for(d=0;d<99;d++){
    if(chamados[d]==q_mesa) return (d);  //estava (d+1)
  }
  return (0xff);//no est na fila
}
//==============================================================================
//   ROTINA PARA MOSTRAR NOS DISPLAIS AS 3 PRIMEIRAS MESAS DA FILA
//==============================================================================
void mostrar_fila(void){
  uint8_t d,f,digx1,dig1a,dig0a;
  
  //testa se est em modo de programao e se sim, sai da rotina
  if(flags.flag_mp==1)return;//esta rotina no deve operar em modo de programao
  
  //obtm os nmero das mesas prioritarias ao atendimento (1,2 e 3)
  digx1=0;
  f=chamados[0];//busca nmero da mesa na primeira posio da fila
  if(f==0){//a fila no tem nenhuma mesa pedindo garom
    dig4=26;//'-' mostra apenas traos
	dig5=26;//'-'
  }
  else{//mostra nmero da mesa mais prioritria ao atendimento
    while(f>=10){ f=f-10; digx1++;if(digx1>9)digx1=0;}//converte decimal  
    dig4=f;//unidade
    dig5=digx1;//dezena
  }  
  
  f=chamados[1];//busca nmero da mesa na segunda posio da fila
  if(f==0){//se no tem mais de uma mesa pedindo garom
    dig2=26;//'-'
	dig3=26;//'-'
  }
  else{//mostra nmero da 2 mesa mais prioritria ao atendimento
    digx1=0;  
    while(f>=10){ f=f-10; digx1++;if(digx1>9)digx1=0;}//converte decimal
    dig2=f;//unidade
    dig3=digx1;//dezena
  }
  
  
  f=chamados[2];//busca nmero da mesa na terceira posio da fila
  if(f==0){// se no tem mais de 2 mesas pedindo garom
    dig0=26;//'-'
	dig1=26;//'-'
  }
  else{//mostra nmero da 3 mesa mais prioritria ao atendimento
    digx1=0;  
    while(f>=10){ f=f-10; digx1++;if(digx1>9)digx1=0;}
    dig0=f;//unidade
    dig1=digx1;//dezena
  }  
  
  dig1a=dig1;//salva dgitos da 3 posio da fila
  dig0a=dig0;
	
	
  if(num_mesa != 0xff ){//mesa tem que estar entre 1 e 99
    //mostra mesa atendida (retirada da fila)  
    if(flags.flag_out==1){ //testa flag de retirada da fila
       f=num_mesa;//pega nmero da mesa     
       digx1=0;//zera
       while(f>=10){ f=f-10; digx1++;if(digx1>9)digx1=0;}//converte a decimal
	   dly_tom3();//gera som 3
       dly_tom2();//gera som 2
       dly_tom1();//gera som 1

       if(pos_fila>1){//pisca dig0/1
          for(d=0;d<3;d++){//pisca digitos 0 e 1 intercalando com traos
             dig0=f;//mostra nmero da mesa (unidades)
             dig1=digx1;//mostra nmero da mesa (dezenas)
             dly_pisca_disp();//delay
             dig0=26;dig1=26;//mostra '-' traos
             dly_pisca_disp();//delay
          } 
	    }
	    else if(pos_fila==1){//pisca dig2/3
	      for(d=0;d<3;d++){//pisca digitos 0 e 1 intercalando com traos
             dig2=f;//mostra nmero da mesa (unidades)
             dig3=digx1;//mostra nmero da mesa (dezenas)          
		     dly_pisca_disp();//delay
             dig2=26;dig3=26;//mostra '-' traos
             dly_pisca_disp();//delay
         } 	   
	   
	   }
	   else {//pisca dig4/5
	       for(d=0;d<3;d++){//pisca digitos 0 e 1 intercalando com traos
             dig4=f;//mostra nmero da mesa (unidades)
             dig5=digx1;//mostra nmero da mesa (dezenas)          
		     dly_pisca_disp();//delay
             dig4=26;dig5=26;//mostra '-' traos
             dly_pisca_disp();//delay
         } 	 
	   }
	   
	 
	   //retira mesa da fila de espera por atendimento
	  if(flags.flag_out==1 && pos_fila<=99){ //tirar mesa da fila pois j foi atendida   
	 		
	    for(d=pos_fila;d<=99;d++){
	      f=chamados[d+1];
	      chamados[d]=f;	  
	      if(f==0)break;//se '0' ento no tem mais controles a frente
	    } 
        if(pont_fila>0)pont_fila--;//decrementa ponteira da fila
      }    
	   
    }//if(flags.flag_out==1){ //testa flag de retirada da fila
	
	
    //mostra mesa que chamou o garom (acrescentada a  fila) 
   if(flags.flag_nm==1){//testa flag de 'nova mesa'entrou na fila
     flags.flag_nm=0;//reseta flag
     f=num_mesa;//obtm o nmero da nova mesa que entrou na fila  
     digx1=0;//zera
     while(f>=10){ f=f-10; digx1++;if(digx1>9)digx1=0;}//converte a decimal 
      dly_tom1();
      dly_tom2();
      dly_tom3();   
	 //gera efeito de piscada para indicar que uma mesa nova entrou na fila
     
       if(dig1 != 26){///3
         for(d=0;d<3;d++){//pisca digitos 0 e 1 intercalando com traos
           dig0=f;//mostra nmero da mesa (unidades)
           dig1=digx1;//mostra nmero da mesa (dezenas)
           dly_pisca_disp();//delay
           dig0=26;dig1=26;//mostra '-' traos
           dly_pisca_disp();//delay
         } 
	   }
	   else if(dig3 != 26){//5
	     for(d=0;d<3;d++){//pisca digitos 0 e 1 intercalando com traos
           dig2=f;//mostra nmero da mesa (unidades)
           dig3=digx1;//mostra nmero da mesa (dezenas)          
		   dly_pisca_disp();//delay
           dig2=26;dig3=26;//mostra '-' traos
           dly_pisca_disp();//delay
         } 	   
	   
	   }
	   else {
	       for(d=0;d<3;d++){//pisca digitos 0 e 1 intercalando com traos
           dig4=f;//mostra nmero da mesa (unidades)
           dig5=digx1;//mostra nmero da mesa (dezenas)          
		   dly_pisca_disp();//delay
           dig4=26;dig5=26;//mostra '-' traos
           dly_pisca_disp();//delay
         } 	 
	   } 
   }
  }//if(num_mesa!= 0xff){
  dig1=dig1a; dig0=dig0a;//retorna valor salvo da 3 posio da fila
  flags.flag_in=0;//reseta flags
  flags.flag_out=0; 
  num_mesa=0xff; //inutiliza nmero da mesa at nova recepo
  pos_fila=0xff;//inutiliza posio da fila
}
//==============================================================================
//   ROTINA DE PROCURA POR CONTROLE REMOTO NA EEPROM, SE CADASTRADO
//==============================================================================
uint8_t controle_foi_cadastrado(void){
   uint16_t b; 
   uint8_t e;
  //EEPROM INTERNA
  for(b=0;b<298;b+=3){//<298 pois 99 * 3 = 297
    e=buffer>>16;
    if(e != EEPROM_Read(b)){continue;}
	e=buffer>>8;
    if(e!= EEPROM_Read(b+1)){continue;}
	e=buffer;
	if(e != EEPROM_Read(b+2)){continue;}
	//return (b/3+1);//tem o controle j cadastrado e retorna o nmero da mesa
	return (b/3+1);//tem o controle j cadastrado e retorna o nmero da mesa           xxx//esta b/3
  }
  
  // EEPROM EXTERNA
  for(b=0;b<298;b+=3){
    
	multiplexa();
	e=buffer>>16;
    if(e != EEPROM_24C08_RdSingle(b)){continue;}
	multiplexa();
	e=buffer>>8;
    if(e != EEPROM_24C08_RdSingle(b+1)){continue;}
	multiplexa();
	e=buffer;
	if(e != EEPROM_24C08_RdSingle(b+2)){continue;}
	
	return (b/3+1);//tem o controle j cadastrado e retorna o nmero da mesa
  }
  //
  return 0xff;//no tem este controle de mesa cadastrado, retorna '0'
 
  
}
//==============================================================================
//   ZERA REGISTRADORES APS RECEPO DE UM BIT
//==============================================================================
void reset_rx (void){ lc=0; hc=0; flags.flag_h=0;  }


//==============================================================================
//          Rotina de recepo RF
//==============================================================================

 void recebe(){
 uint8_t g;
  if(flags.flag_rok)return;
  if(PINB & (1 << rfin)){ flags.flag_h=1; if(++hc==0) --hc; }
  
  else {
       if(!flags.flag_h){if(++lc==0) --lc;}  // incrementa lc e limita 0xff
        else  {
          if(lc>tmax) {reset_rx (); qb=QBIT;}
         
          else {
              buffer= buffer>>1;
              if(lc>hc)buffer=buffer | 0x80000000;
              if (--qb==0){
                qb=QBIT; //salva botes
				buffer=buffer>>4;
				if(buffer>>24 == 0x0a){//testa se recebeu o anticode 1010
				  g=(buffer>>16) & 0b11000000;//pega os botes
				  if(g==0b10000000){flags.flag_in=1;flags.flag_out=0;}//entra na fila
                  if(g==0b01000000) {flags.flag_out=1;flags.flag_in=0;}//sai da fila 
				  buffer= buffer & 0x003fffff;//apaga botes
                  flags.flag_rok=1;}  // h uma recepo no buffer
				}
                reset_rx();
          }
       }
   }
    dly_us(50);
}


/*******************************************************************************
    Rotina  para Multiplexao dos Displays
*******************************************************************************/
 void multiplexa(){
 //static unsigned char vez;
   if(++vez>5)vez=0;
   
   #ifdef  catodo_comum
   PORTD=0; //apaga seguimentos e disp5   
   #else 
   PORTD=0x7f; //apaga seguimentos e disp5 
   #endif 
   PORTC &=~c0disp0;//reseta displais 0 a 4
   PORTC &=~c1disp1;
   PORTC &=~c2disp2;
   PORTC &=~c3disp3;
   PORTB &=~b2disp4;

   switch(vez){
      case 0: {
	     #ifdef  catodo_comum
         PORTD= PORTD | num[dig0];
		 #else
		 PORTD= PORTD & ~num[dig0];
		 #endif
         PORTC |=c0disp0;//seta disp0         
         break;
      }
      case 1: {
         #ifdef  catodo_comum
         PORTD= PORTD | num[dig1];
		 #else
		 PORTD= PORTD & ~num[dig1];
		 #endif
         PORTC |=c1disp1;//seta disp1         
         break;
      }
      case 2: {
         #ifdef  catodo_comum
         PORTD= PORTD | num[dig2];
		 #else
		 PORTD= PORTD & ~num[dig2];
		 #endif
         PORTC |=c2disp2;//seta disp2         
         break;
      }
      case 3: {
         #ifdef  catodo_comum
         PORTD= PORTD | num[dig3];
		 #else
		 PORTD= PORTD & ~num[dig3];
		 #endif
         PORTC |=c3disp3;//seta disp3         
         break;
      }
      case 4: {
         #ifdef  catodo_comum
         PORTD= PORTD | num[dig4];
		 #else
		 PORTD= PORTD & ~num[dig4];
		 #endif
         PORTB |=b2disp4;//seta disp0         
         break;
      }
      case 5: {
         #ifdef  catodo_comum
         PORTD= PORTD | num[dig5];
		 #else
		 PORTD= PORTD & ~num[dig5];
		 #endif
         PORTD |=d7disp5;//seta disp5         
         break;
      }
   } 

}



// Performs 24C08 Init
void EEPROM_24C08_Init() {//aprox. 100khz baudrate
	TWSR=0;//0
	TWBR = (F_CPU / 100000UL - 16) / 2;//for F_cpu >3mhz .if<3mhz use TWBR =10
}

// Writes data to 24C08 EEPROM - single location
void EEPROM_24C08_WrSingle(uint16_t wAddr, uint8_t wData) {
  uint8_t d;
    
    d = wAddr>>8; //pega o byte high do endereo
    d = d & 7; //pega apenas bits 0 a 2
    d=d<<1; //coloca nos bits 3 a 1 o endereo do bloco
    d = d | 0xA0; //coloca tipo de equipamento (A=memoria serial)	
    TWI_Start();  // issue Twi start signal
    TWI_Write(d); // send byte via Twi  (command to 24cO8)SLA *
    TWI_Write(wAddr); // send byte (address of EEPROM location)
    TWI_Write(wData); // send data (data to be written)
	TWI_Stop();   //stop signal
	dly_write();//delay de 10 msegundos para escrita
}

// Reads data from 24C08 EEPROM - single location (random)
uint8_t EEPROM_24C08_RdSingle(uint16_t rAddr) {
    uint8_t reslt,a;
	
    a = rAddr>>8;
    a = a & 7;
    a=a<<1;//coloca nos bits 3 a 1 o endereo do bloco
    a = a | 0xA0;
    TWI_Start();                // issue Twi start signal
    TWI_Write(a);            // send byte via Twi  (device address + BANK + BIT WRITE (0))
    TWI_Write(rAddr);           // send byte (data address)
    TWI_Start();                // issue Twi signal repeated start

    TWI_Write(a+1);            // send byte (device address + BANK + BIT READ (1))
    reslt = TWI_Read(0);       // Read the data (NO acknowledge)
    while (TWI_Busy());
    asm volatile ("nop");                  // Wait for the read cycle to finish
    TWI_Stop();
    return reslt;
}

//Envia Start Bit na linha TWI
void TWI_Start(){
  TWCR = (1<< TWINT) | (1<< TWSTA) | (1<<TWEN);//start
  while(!(TWCR & (1<<TWINT)));//verifica se j transmitiu 'start'
}

//Verifica de TWI est ocupado
 uint8_t TWI_Busy(void){  
    if((TWSR & 0XF0) ==0X50)return 0; else return 0xff;    
 }


//Envia endereo  pelo mdulo TWI
void TWI_Write(uint8_t dataddress){
 TWDR=dataddress;
 TWCR= (1<<TWINT) | (1<<TWEN);
 while(!(TWCR & (1<<TWINT)));//verifica se j transmitiu 'DATA/ADDRESS'
}

//L dado recebido via mdulo TWI
uint8_t TWI_Read(uint8_t b){
  TWCR = _BV(TWINT) | _BV(TWEN) ;//no ack (ack=| _BV(TWEA));
  while ((TWCR & _BV(TWINT)) == 0) ;//espera fim da transmisso
  return TWDR;
} 


//Envia Stop bit na linha TWI
void TWI_Stop(void){ 
  TWCR = (1<< TWINT) | (1<< TWSTO) | (1<<TWEN);//stop  
}

//Delay para concluso de gravao da EEprom Externa de aprox. 10mseg
void dly_write(void){
 uint8_t c;
 for(c=0;c<40;c++)dly_us(255);//delay para executar stop no barramento i2c
}


/**************************************************
     Delay em Geral
**************************************************/
void dly_us(uint8_t utime){
 uint8_t a,b; 
  for(b=0;b<utime;b++){
    a=8;
    while(a>0) a--;
  }
}

void dly_ms(uint8_t utime){
 uint8_t a,b,c; 
  for(b=0;b<=utime;b++){
    for(c=0;c<4;c++){
	  a=4;
      dly_us(250);
    }
  }

}

//delay para o tempo da piscada dos displais
void dly_msg(void){
 uint16_t  v;
   for(v=0; v<500; v++){
		  multiplexa();
		  dly_ms(4);
   }
}

//delay para o tempo da piscada dos displais
void dly_pisca_disp(void){
 uint8_t  v;
   for(v=0; v<100; v++){
		  multiplexa();
		  dly_ms(4);
   }
}

//Rotina geradora de Aviso Sonoro 1
void dly_tom1(void){
 uint8_t  v,y,w;
 
   for(v=0; v<72; v++){
		
		for(y=0;y<17;y++){ 
		    w=250;
		    while(w)w--;          
		  }
		  PORTB = PORTB | (1<<PB5);
		  multiplexa();
		  for(y=0;y<17;y++){ 		  
		    w=250;
		    while(w)w--;
		  }
		  PORTB = PORTB & (!(1<<PB5));
		  multiplexa();
		
   }
}

//Rotina geradora de Aviso Sonoro 2
void dly_tom2(void){
 uint8_t  v,y,w;
 
    for(v=0; v<67; v++){
		
		for(y=0;y<22;y++){ 
		    w=250;
		    while(w)w--;          
		  }
		  PORTB = PORTB | (1<<PB5);
		  multiplexa();
		  for(y=0;y<22;y++){ 		  
		    w=250;
		    while(w)w--;
		  }
		  PORTB = PORTB & (!(1<<PB5));
		  multiplexa();
		
   }
}

//Rotina geradora de Aviso Sonoro 3
void dly_tom3(void){
 uint8_t  v,y,w;
 
    for(v=0; v<62; v++){
		
		for(y=0;y<27;y++){ 
		    w=250;
		    while(w)w--;          
		  }
		  PORTB = PORTB | (1<<PB5);
		  multiplexa();
		  for(y=0;y<27;y++){ 		  
		    w=250;
		    while(w)w--;
		  }
		  PORTB = PORTB & (!(1<<PB5));
		  multiplexa();
		
   }
}

// ************************* fim do programa*****************************